"""
A simple data collector that saves all input into a csv file.

"""
import pandas as pd
import mosaik_api_v3 as mosaik_api
import numpy as np

META = {
    'type': 'event-based',
    'models': {
        'CSVWriter': {
            'public': True,
            'any_inputs': True,
            'params': ['buff_size', 'attrs'],
            'attrs': [],
        },
    },
}


class CSVWriter(mosaik_api.Simulator):
    def __init__(self):
        super().__init__(META)
        self.eid = None
        self.time_resolution = None
        self.date_format = None
        self.start_date = None
        self.output_file = None
        self.df = None
        self.buff_size = None
        self.attrs = None
        self.nan_representation = None

    def init(self, sid, time_resolution, start_date,
             date_format='%Y-%m-%d %H:%M:%S', output_file='results.csv',
             nan_rep='NaN'):
        self.time_resolution = time_resolution
        self.date_format = date_format
        self.start_date = pd.to_datetime(start_date, format=date_format)
        self.output_file = output_file
        self.nan_representation = nan_rep
        self.df = pd.DataFrame([])

        return self.meta

    def create(self, num, model, buff_size=500, attrs=None):
        if num > 1 or self.eid is not None:
            raise RuntimeError('Can only create one instance of CSVWriter.')
        if attrs:
            self.attrs = ['date']
            self.attrs.extend(attrs)
        self.buff_size = buff_size
        self.eid = 'CSVWriter'

        return [{'eid': self.eid, 'type': model}]

    def step(self, time, inputs, max_advance):
        current_date = (self.start_date
                        + pd.Timedelta(time * self.time_resolution, unit='seconds'))

        data_dict = {'date': current_date}
        for attr, values in inputs.get(self.eid, {}).items():
            for src, value in values.items():
                data_dict[f'{src}-{attr}'] = [value]
        if self.attrs:
            # List of attributes was provided in create() function or created in first step
            # Based on this list the DataFrame is created. Thus, also attributes with no input
            # are in the Dataframe as NaN values.
            df_data = pd.DataFrame(data_dict, columns=self.attrs)
            df_data.set_index('date', inplace=True)
        else:
            # Initialize attribute list based on the attributes in the first step
            # This might be problematic, if in a later step more attributes are provided
            self.attrs = list(data_dict.keys())
            df_data = pd.DataFrame(data_dict)
            df_data.set_index('date', inplace=True)

        if time == 0:
            self.df = df_data
            self.df.to_csv(self.output_file,
                           mode='w',
                           header=True,
                           date_format=self.date_format,
                           na_rep=self.nan_representation)
            self.df = pd.DataFrame([])
        elif time > 0:
            self.df = pd.concat([self.df, df_data])
            if time % self.buff_size == 0:
                self.df.to_csv(self.output_file,
                               mode='a',
                               header=False,
                               date_format=self.date_format,
                               na_rep=self.nan_representation)
                self.df = pd.DataFrame([])

        return None

    def finalize(self):
        self.df.to_csv(self.output_file,
                       mode='a',
                       header=False,
                       date_format=self.date_format,
                       na_rep=self.nan_representation)


if __name__ == '__main__':
    mosaik_api.start_simulation(CSVWriter())



import pandas as pd

import mosaik_api_v3 as mosaik_api

DATE_FORMAT = 'YYYY-MM-DD HH:mm:ss'

SENTINEL = object()


class CSV(mosaik_api.Simulator):
    def __init__(self):
        super().__init__({'models': {}})
        self.start_date = None
        self.data = None
        self.attrs = None
        self.cache = None
        self.sid = None
        self.eid = None
        self.eids = []
        self.delimiter = None
        self.type = None
        self.time_res = None
        self.next_date = None
        self.model_name = None
        self.next_index = None
        
    def init(self, sid, time_resolution, sim_start, datafile, date_format=None, type="time-based", delimiter=','):
        self.type = type
        if self.type != "time-based" and self.type != "event-based":
            print("Please enter the correct type. The type can either be time-based or event-based")
        self.sid = sid
        self.time_res = pd.Timedelta(time_resolution, unit='seconds')
        start_date = self.start_date = pd.to_datetime(sim_start, format=date_format)
        self.next_date = self.start_date
        self.delimiter = delimiter

        # Check if first line is the header with column names (our attributes)
        # or a model name:
        with open(datafile) as f:
            first_line = f.readline()
        first_line = first_line.strip('\n')

        if len(first_line.split(self.delimiter)) == 1:
            self.model_name = first_line
            header = 1
        else:
            header = 0
            self.model_name = 'Data'

        data = self.data = pd.read_csv(datafile, index_col=0, parse_dates=True,
                                       header=header)
        data.rename(columns=lambda x: x.strip(), inplace=True)

        self.attrs = [attr.strip() for attr in data.columns]

        for i, attr in enumerate(self.attrs):
            try:
                # Try stripping comments
                attr = attr[:attr.index('#')]
            except ValueError:
                pass
            self.attrs[i] = attr.strip()

        # Rename column names of datafrmae
        data.columns = self.attrs

        self.meta['type'] = self.type

        self.meta['models'][self.model_name] = {
            'public': True,
            'params': [],
            'attrs': self.attrs,
        }

        # Find first relevant value:
        if self.type == "time-based":
            first_index = data.index.get_indexer([start_date], method='ffill')[0]
            self.next_index = first_index
        else:
            first_index = data.index.get_indexer([start_date], method='bfill')[0]
            first_date = data.index[first_index]
            if first_date == start_date:
                self.next_index = first_index
            else:
                self.next_index = -1

        return self.meta

    def create(self, num, model):
        if model != self.model_name:
            raise ValueError('Invalid model "%s" % model')

        start_idx = len(self.eids)
        entities = []
        for i in range(num):
            eid = '%s_%s' % (model, i + start_idx)
            entities.append({
                'eid': eid,
                'type': model,
                'rel': [],
            })
            self.eids.append(eid)
        return entities

    def step(self, time, inputs, max_advance):
        data = self.data
        if self.next_index >= 0:
            current_row = data.iloc[self.next_index]
            self.cache = dict(current_row)
        else:
            self.cache = {}
        self.next_index += 1
        try:
            next_date = self.data.index[self.next_index]
            next_step = int((next_date - self.start_date)/self.time_res)
        except IndexError:
            next_step = max_advance

        return next_step

    def get_data(self, outputs):
        data = {}
        for eid, attrs in outputs.items():
            if eid not in self.eids:
                raise ValueError('Unknown entity ID "%s"' % eid)
            data[eid] = {}
            for attr in attrs:
                data[eid][attr] = self.cache[attr]

        return data


def main():
    return mosaik_api.start_simulation(CSV(), 'mosaik-csv simulator')


if __name__ == '__main__':
    main()



from setuptools import setup


setup(
    name='mosaik-csv',
    version='2.0.0',
    author='mosaik development team',
    author_email='mosaik@offis.de',
    description=('Presents CSV datasets to mosaik as models.'),
    long_description=(open('README.rst').read() + '\n\n' +
                      open('CHANGES.txt').read() + '\n\n' +
                      open('AUTHORS.txt').read()),
    url='https://gitlab.com/mosaik/components/data/mosaik-csv',
    py_modules=['mosaik_csv',
                'mosaik_csv_writer'],
    install_requires=[
        'arrow>=1.0.0',
        'mosaik-api-v3',
        'importlib-metadata<5.0',
        'pandas',
    ],
    include_package_data=True,
    entry_points={
        'console_scripts': [
            'mosaik-csv = mosaik_csv.mosaik:main',
        ],
    },
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: GNU Lesser General Public License v2 (LGPLv2)',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'Topic :: Scientific/Engineering',
    ],
)
